@@ -20,10 +20,8 @@ class RawStenciledFile(io.RawIOBase):
 
     def __init__(
         self,
-        fileobj: Optional[IO] = None,
-        stencils: Optional[List[Tuple[int, int]]] = None,
-        fileobjLock=None,
-        fileStencils: Optional[List[Tuple[IO, int, int]]] = None,
+        fileStencils: List[Tuple[IO, int, int]],
+        fileObjectLock=None,
     ) -> None:
         """
         stencils: A list tuples specifying the offset and length of the underlying file to use.
@@ -45,32 +43,15 @@ class RawStenciledFile(io.RawIOBase):
             stencil = [(0,3),(0,3)]
                 Make a 6B size file containing the first 3B of fileobj twice concatenated together.
         """
+        io.RawIOBase.__init__(self)
 
         self.offset = 0
-        self.fileobjLock = fileobjLock
+        self.fileObjectLock = fileObjectLock
         self.offsets: List[int] = []
         self.sizes: List[int] = []
         self.fileObjects: List[IO] = []
 
-        # Convert stencils to internal format
-        if fileStencils:
-            if stencils or fileobj:
-                raise ValueError(
-                    "Either the deprecated combination of fileobj and stencils or the new interface using "
-                    "fileStencils may be specified, not both!"
-                )
-            self.fileObjects, self.offsets, self.sizes = zip(*fileStencils)
-        else:
-            if stencils:
-                if not fileobj:
-                    raise ValueError("Stencils may not be specified without a valid file object!")
-
-                self.offsets = [x[0] for x in stencils]
-                self.sizes = [x[1] for x in stencils]
-                self.fileObjects = [fileobj] * len(self.sizes)
-            else:
-                self.offsets = []
-                self.fileObjects = []
+        self.fileObjects, self.offsets, self.sizes = zip(*fileStencils)
 
         # Check whether values make sense
         for offset in self.offsets:
@@ -142,6 +123,7 @@ class RawStenciledFile(io.RawIOBase):
 
     @overrides(io.RawIOBase)
     def readinto(self, buffer):
+        """Generic implementation which uses read."""
         with memoryview(buffer) as view, view.cast("B") as byteView:  # type: ignore
             readBytes = self.read(len(byteView))
             byteView[: len(readBytes)] = readBytes
@@ -159,7 +141,7 @@ class RawStenciledFile(io.RawIOBase):
         if i >= len(self.sizes):
             return result
 
-        with self.fileobjLock if self.fileobjLock else _DummyContext():
+        with self.fileObjectLock if self.fileObjectLock else _DummyContext():
             # Note that seek and read of the file object itself do not seem to check against this and
             # instead lead to a segmentation fault in the multithreading tests.
             if self.fileObjects[i].closed:
@@ -199,15 +181,156 @@ class RawStenciledFile(io.RawIOBase):
         return self.offset
 
 
+class RawJoinedFileFromFactory(io.RawIOBase):
+    def __init__(self, file_object_factories: List[Callable[[], IO[bytes]]], file_lock=None) -> None:
+        """
+        Similar to StenciledFile but instead of joining a list of file objects, which neccessitates keeping all
+        files open, this class opens each file on demand and only keeps one file open. This is useful to avoid
+        exceeding the system limit for the number of opened I/O handles.
+        """
+        io.RawIOBase.__init__(self)
+
+        self.offset = 0
+        self.fileObjectLock = file_lock
+        # Stores index and file object of currently opened file object. It basically is a cache for the factories.
+        self.fileObject: Optional[Tuple[int, IO[bytes]]] = None
+
+        self.sizes: List[int] = []
+        self.factories: List[Callable[[], IO[bytes]]] = []
+        self.cumsizes = [0]
+        self._seekable = True
+        for factory in file_object_factories:
+            with factory() as file:
+                if not file.seekable():
+                    self._seekable = False
+                if not file.readable():
+                    raise ValueError("All file objects to be joined must be readable")
+
+                size = file.seek(0, io.SEEK_END)
+                if size is None or size < 0:
+                    raise ValueError(f"Failed to query size for factory: {factory}")
+
+                # Filter out zero-sized regions (or else we would have to skip them inside 'readinto' in order to not
+                # return an empty reply even though the end of file has not been reached yet!)
+                if size > 0:
+                    self.sizes.append(size)
+                    self.factories.append(factory)
+                    self.cumsizes.append(self.cumsizes[-1] + size)
+
+    def _findStencil(self, offset: int) -> int:
+        # See StenciledFile._findStencil
+        assert offset >= 0
+        i = bisect.bisect_left(self.cumsizes, offset + 1) - 1
+        assert i >= 0
+        return i
+
+    def _getFileObject(self, index: int) -> IO[bytes]:
+        if self.fileObject and self.fileObject[0] == index:
+            return self.fileObject[1]
+
+        self.close()
+        self.fileObject = index, self.factories[index]()
+        return self.fileObject[1]
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, exception_type, exception_value, exception_traceback):
+        self.close()
+
+    @overrides(io.RawIOBase)
+    def close(self) -> None:
+        if self.fileObject and not self.fileObject[1].closed:
+            self.fileObject[1].close()
+
+    @overrides(io.RawIOBase)
+    def fileno(self) -> int:
+        # This is a virtual Python level file object and therefore does not have a valid OS file descriptor!
+        raise io.UnsupportedOperation()
+
+    @overrides(io.RawIOBase)
+    def seekable(self) -> bool:
+        return self._seekable
+
+    @overrides(io.RawIOBase)
+    def readable(self) -> bool:
+        return True
+
+    @overrides(io.RawIOBase)
+    def writable(self) -> bool:
+        return False
+
+    @overrides(io.RawIOBase)
+    def readinto(self, buffer):
+        """Generic implementation which uses read."""
+        with memoryview(buffer) as view, view.cast("B") as byteView:  # type: ignore
+            readBytes = self.read(len(byteView))
+            byteView[: len(readBytes)] = readBytes
+        return len(readBytes)
+
+    @overrides(io.RawIOBase)
+    def read(self, size: int = -1) -> bytes:
+        # See StenciledFile.read
+        if size == -1:
+            size = self.cumsizes[-1] - self.offset
+
+        # This loop works in a kind of leapfrog fashion. On each even loop iteration it seeks to the next stencil
+        # and on each odd iteration it reads the data and increments the offset inside the stencil!
+        result = b''
+        i = self._findStencil(self.offset)
+        if i >= len(self.sizes):
+            return result
+
+        with self.fileObjectLock if self.fileObjectLock else _DummyContext():
+            fileObject = self._getFileObject(i)
+
+            # Note that seek and read of the file object itself do not seem to check against this and
+            # instead lead to a segmentation fault in the multithreading tests.
+            if fileObject.closed:
+                raise ValueError("A closed file can't be read from!")
+
+            offsetInsideStencil = self.offset - self.cumsizes[i]
+            assert offsetInsideStencil >= 0
+            assert offsetInsideStencil < self.sizes[i]
+            fileObject.seek(offsetInsideStencil, io.SEEK_SET)
+
+            # Read as much as requested or as much as the current contiguous region / stencil still contains
+            readableSize = min(size, self.sizes[i] - (self.offset - self.cumsizes[i]))
+            tmp = fileObject.read(readableSize)
+            self.offset += len(tmp)
+            result += tmp
+
+        return result
+
+    @overrides(io.RawIOBase)
+    def seek(self, offset: int, whence: int = io.SEEK_SET) -> int:
+        """Generic implementation that simply updates self.offset nothing else."""
+        if whence == io.SEEK_CUR:
+            self.offset += offset
+        elif whence == io.SEEK_END:
+            self.offset = self.cumsizes[-1] + offset
+        elif whence == io.SEEK_SET:
+            self.offset = offset
+
+        if self.offset < 0:
+            raise ValueError("Trying to seek before the start of the file!")
+        if self.offset >= self.cumsizes[-1]:
+            return self.offset
+
+        return self.offset
+
+    @overrides(io.RawIOBase)
+    def tell(self) -> int:
+        return self.offset
+
+
 class StenciledFile(io.BufferedReader):
     def __init__(
         self,
-        fileobj: Optional[IO] = None,
-        stencils: Optional[List[Tuple[int, int]]] = None,
-        fileobjLock=None,
-        fileStencils: Optional[List[Tuple[IO, int, int]]] = None,
+        fileStencils: List[Tuple[IO, int, int]],
+        fileObjectLock=None,
     ) -> None:
-        super().__init__(RawStenciledFile(fileobj, stencils, fileobjLock, fileStencils))
+        super().__init__(RawStenciledFile(fileStencils, fileObjectLock))
 
 
 class JoinedFile(io.BufferedReader):
@@ -218,7 +341,19 @@ class JoinedFile(io.BufferedReader):
                 raise ValueError("Failed to query size of file object:", fobj)
 
         fileStencils = [(fobj, 0, size if size else 0) for fobj, size in zip(file_objects, sizes)]
-        super().__init__(RawStenciledFile(fileStencils=fileStencils, fileobjLock=file_lock), buffer_size=buffer_size)
+        super().__init__(RawStenciledFile(fileStencils=fileStencils, fileObjectLock=file_lock), buffer_size=buffer_size)
+
+
+class JoinedFileFromFactory(io.BufferedReader):
+    def __init__(
+        self, file_object_factories: List[Callable[[], IO]], file_lock=None, buffer_size: int = io.DEFAULT_BUFFER_SIZE
+    ) -> None:
+        """
+        Similar to JoinedFile but instead of joining a list of file objects, which neccessitates keeping all
+        files open, this class opens each file on demand and only keeps one file open. This is useful to avoid
+        exceeding the system limit for the number of opened I/O handles.
+        """
+        super().__init__(RawJoinedFileFromFactory(file_object_factories, file_lock), buffer_size=buffer_size)
 
 
 class LambdaReaderFile(io.RawIOBase):
@@ -226,6 +361,7 @@ class LambdaReaderFile(io.RawIOBase):
 
     def __init__(self, rawRead: Callable[[int, int], bytes], size: int) -> None:
         """rawRead: Function which returns bytes for (offset, size) input."""
+        io.RawIOBase.__init__(self)
 
         self.offset = 0
         self.rawRead = rawRead
