@@ -7,25 +7,40 @@ import traceback
 
 from typing import IO, Union
 
-from .compressions import supportedCompressions, rarfile, zipfile
+from .compressions import supportedCompressions, checkForSplitFile, rarfile, zipfile
 from .utils import CompressionError, RatarmountError
 from .MountSource import MountSource
 from .FolderMountSource import FolderMountSource
 from .RarMountSource import RarMountSource
+from .SingleFileMountSource import SingleFileMountSource
 from .SQLiteIndexedTar import SQLiteIndexedTar
+from .StenciledFile import JoinedFileFromFactory
 from .ZipMountSource import ZipMountSource
 
 
 def openMountSource(fileOrPath: Union[str, IO[bytes]], **options) -> MountSource:
     printDebug = int(options.get("printDebug", 0)) if isinstance(options.get("printDebug", 0), int) else 0
 
+    joinedFileName = ''
     if isinstance(fileOrPath, str):
         if not os.path.exists(fileOrPath):
-            raise Exception("Mount source does not exist!")
+            raise RatarmountError(f"Mount source does not exist: {fileOrPath}")
 
         if os.path.isdir(fileOrPath):
             return FolderMountSource('.' if fileOrPath == '.' else os.path.realpath(fileOrPath))
 
+        splitFileResult = checkForSplitFile(fileOrPath)
+        if splitFileResult:
+            filesToJoin = splitFileResult[0]
+            joinedFileName = os.path.basename(filesToJoin[0]).rsplit('.', maxsplit=1)[0]
+            if 'indexFilePath' not in options or not options['indexFilePath']:
+                options['indexFilePath'] = filesToJoin[0] + ".index.sqlite"
+            # https://docs.python.org/3/faq/programming.html
+            # > Why do lambdas defined in a loop with different values all return the same result?
+            fileOrPath = JoinedFileFromFactory(
+                [(lambda file=file: open(file, 'rb')) for file in filesToJoin]  # type: ignore
+            )
+
     try:
         if 'rarfile' in sys.modules and rarfile.is_rarfile(fileOrPath):
             return RarMountSource(fileOrPath, **options)
@@ -57,7 +72,7 @@ def openMountSource(fileOrPath: Union[str, IO[bytes]], **options) -> MountSource
         if hasattr(fileOrPath, 'seek'):
             fileOrPath.seek(0)  # type: ignore
 
-    if 'zipfile' in sys.modules:
+    if 'zipfile' in sys.modules and zipfile is not None:
         try:
             # is_zipfile is much too lax when testing for ZIPs because it's only testing for the central directory
             # at the end of the file not the magic bits at the beginning. Meaning, if another non-ZIP archive has
@@ -85,4 +100,7 @@ def openMountSource(fileOrPath: Union[str, IO[bytes]], **options) -> MountSource
             if hasattr(fileOrPath, 'seek'):
                 fileOrPath.seek(0)  # type: ignore
 
+    if joinedFileName and not isinstance(fileOrPath, str):
+        return SingleFileMountSource(joinedFileName, fileOrPath)
+
     raise CompressionError(f"Archive to open ({str(fileOrPath)}) has unrecognized format!")
